--[[

Version 0.01 - Initial release of beta code to WowAce SVN

Author : Levva of EU Khadgar
	
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local AceAddon = LibStub("AceAddon-3.0")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
PriorityQueue = AceAddon:NewAddon("PriorityQueue", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local REVISION = tonumber(("$Revision: 2 $"):match("%d+"))
local _, _, _, clientVersion = GetBuildInfo()
if clientVersion >= 30100 then
	PriorityQueue.patch31 = true
else
	PriorityQueue.patch31 = false
end
PriorityQueue.class = select(2, UnitClass('player'))
PriorityQueue.constants = {}
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.

PriorityQueue.PriorityFrame = CreateFrame("Frame", "PQ_PriorityFrame", UIParent)
PriorityQueue.msgFrame = CreateFrame("MessageFrame","PriorityQueueMsg",UIParent)

PriorityQueue.textures = {}
PriorityQueue.borders = {}
PriorityQueue.fonts = {}
PriorityQueue.sounds = {}
PriorityQueue.frameBackdrop ={
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 12,
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	}
	
PriorityQueue.priorityTable = {}
PriorityQueue.priorityTable.name = {}
PriorityQueue.priorityTable.icon = {}
PriorityQueue.priorityTable.test = {}

-------------------
-- Config details
-------------------

PriorityQueue.defaults = {
	char = {
		message = "Welcome Home!",
		relativeTo = "UIParent",
		relativePoint = "CENTER",
		point = "CENTER",
		texture = "Blizzard",
		msgfont = "Friz Quadrata TT",
		msgfontsize = 24,
		msgfonteffect = "none",
		xOffset = 0,
		yOffset = -100,
		fWidth = 300,
		fHeight = 175,
		resetOn = true,
		scale = 1,
		debug = false,
		msg = { r = 1, g = .5, b = 0, },
		msgtime = 2,
		newsitem = 0,
		colours = {
			watershield = { r=.6, g=.6,	b=1, a=.3 },
			lightningshield = { r=0, g=0, b=1, a=.5 },
			earthshield = { r=0, g=1, b=0, a=.5 },
			noshield = { r=1, g=0, b=0, a=1 },
			
			flameshock = { r=1, g=.6, b=.2, a=.9 },
			earthshock = { r=0, g=1, b=.3, a=.9	},
			frostshock = { r=.6, g=.6, b=1, a=.9 },
			windshock = { r=.6, g=.6, b=.6, a=.9 },
			
			maelstrom = { r=1, g=.5, b=1, a=.3 },
			msalpha = .3,
			msalphaFull = .9,

			lavalash = { r=1, g=.1, b=.1, a=.9 },
			feralspirit = { r=0, g=.6, b=.95, a=.9 },
			feralspiritCD = { r=0, g=.6, b=.95, a=.9 },
			stormstrike = { r=1, g=1, b=.2, a=.9 },
			windfury = { r=1, g=0, b=0, a=.5 },
		},
		priority = {
			show = true,
			titleshow = true,
			relativeTo = "UIParent",
			relativePoint = "CENTER",
			point = "CENTER",
			fWidth = 50,
			fHeight = 50,
			xOffset = -185,
			yOffset = -40,
			alpha = 0.4,
			scale = 1,	
			next = 1,
			cooldown = 0.25,
			fstimeleft = 0.5,
			totemtimeleft = 20,
			srmana = 5,
			wsmana = 0,
			shieldorbs = 1,
			hwhealth = 25,
			chingroup = true,
			worldbossonly = true,
			combopoints = true,
			option = { "mw5_lb", "es_ss", "ss", "es", "ll", "st", "shield", "none",},
		},
		warning = {
			show = true,
			duration = 3,
			shield = false,
			weapon = true,
			range = false,
			timeleft = 300, 
			colour = { r = 1, g = .5, b = 0, a = 0.5, },
			relativeTo = "UIParent",
			relativePoint = "TOP",
			point = "CENTER",
			fWidth = 400,
			fHeight = 75,
			xOffset = 0,
			yOffset = -250,
		},
	}
}

-----------------------------------------
-- Initialisation & Startup Routines
-----------------------------------------

function PriorityQueue:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("PriorityQueueDBPC", PriorityQueue.defaults, "char")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PriorityQueue", self:GetOptions(), {"PriorityQueue", "PQ"} )
	media.RegisterCallback(self, "LibSharedMedia_Registered")
	
	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("PriorityQueue","PriorityQueue")
	self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("PriorityQueue", "PriorityQueue")
	self.db:RegisterDefaults(self.defaults)
	self:CreatePriorityFrame()
	self:CreateMsgFrame()
	self:News()
	local version = GetAddOnMetadata("PriorityQueue","Version")
	self.version = ("PriorityQueue v%s (r%s)"):format(version, REVISION)
	self:Print(self.version.." Loaded.")
end

function PriorityQueue:OnDisable()
    -- Called when the addon is disabled
end

function PriorityQueue:OnEnable()
	self:LibSharedMedia_Registered()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
--	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
--	self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
--	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("UNIT_MANA")
--	self:RegisterEvent("CHARACTER_POINTS_CHANGED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
--	self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	self:RegisterEvent("UI_ERROR_MESSAGE")
end

function PriorityQueue:LibSharedMedia_Registered()
	for k, v in pairs(media:List("statusbar")) do
		self.textures[v] = v
	end
	for k, v in pairs(media:List("border")) do
		self.borders[v] = v
	end
	for k, v in pairs(media:List("font")) do
		self.fonts[v] = v
	end
	for k, v in pairs(media:List("sound")) do
		self.sounds[v] = v
	end
end

----------------------
-- Event Routines
----------------------

function PriorityQueue:PLAYER_ENTERING_WORLD()
	self:SetBorderTexture(nil, self.db.char.border)
	self:RedrawFrames()
end

function PriorityQueue:UI_ERROR_MESSAGE()
	if self.db.char.warning.range then
		if arg1 then self:DebugPrint("error message is :"..arg1) end
		if strfind(arg1, L["Out of range"]) or strfind(arg1, L["too far away"]) then
			self:PrintMsg(arg1, self.db.char.warning.colour, 0.5)
		end
	end
end

function PriorityQueue:SPELL_UPDATE_COOLDOWN()
	if self.db.char.gcdshow then
		self:GCDBar()
	end
end

function PriorityQueue:CHARACTER_POINTS_CHANGED()
	-- probably need to check if player has valid talents for certain priority abilities
end

function PriorityQueue:PLAYER_REGEN_DISABLED() -- Entering Combat
	if self.db.char.priority.show then
		self.db.char.priority.next = "none"
		self:SetPriorityIcon(self.db.char.priority.next)
		self.updatePriority = true
		self.PriorityFrame:Show()
	end
end

function PriorityQueue:PLAYER_REGEN_ENABLED() -- Leaving Combat
	self:SetPriorityIcon("none")
	self.updatePriority = false
	self.PriorityFrame:Hide()
end

function PriorityQueue:PLAYER_TARGET_CHANGED()
	if InCombatLockdown() then
		-- will need to reset priority for new target
	end
end

-------------------------------
-- Priority Frame functions
-------------------------------

function PriorityQueue:SetPriorityBackdrop(icon)
	self.PriorityFrame:SetBackdrop({ bgFile = icon,
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 12,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
end

function PriorityQueue:CreatePriorityFrame()
	self.updatePriority = false
	self.PriorityFrame:SetScale(self.db.char.priority.scale)
	self.PriorityFrame:SetFrameStrata("BACKGROUND")
	self.PriorityFrame:SetWidth(self.db.char.priority.fWidth)
	self.PriorityFrame:SetHeight(self.db.char.priority.fHeight)
	self:SetPriorityBackdrop("Interface/Tooltips/UI-Tooltip-Background")
	self.PriorityFrame:SetBackdropColor(0, 0, 0, self.db.char.priority.alpha);
	self.PriorityFrame:SetMovable(true);
	self.PriorityFrame:RegisterForDrag("LeftButton");
	self.PriorityFrame:SetPoint(self.db.char.priority.point, self.db.char.priority.relativeTo, self.db.char.priority.relativePoint, self.db.char.priority.xOffset, self.db.char.priority.yOffset)
	self.PriorityFrame:SetScript("OnDragStart", 
		function()
			self.PriorityFrame:StartMoving();
		end );
	self.PriorityFrame:SetScript("OnDragStop",
		function()
			self.PriorityFrame:StopMovingOrSizing();
			self.PriorityFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char.priority, self.PriorityFrame);
		end );
	self.PriorityFrame:SetScript("OnUpdate", 
		function()
			if self.updatePriority then
				PriorityQueue.db.char.priority.previous = PriorityQueue.db.char.priority.next
				PriorityQueue:SetNextPriority()
				if PriorityQueue.db.char.priority.next ~= PriorityQueue.db.char.priority.previous or PriorityQueue:CheckShamanisticRage() then
					PriorityQueue:SetPriorityIcon(PriorityQueue.db.char.priority.next)
				end
			end
		end );
		
	if not self.PriorityFrame.topText then
		self.PriorityFrame.topText = self.PriorityFrame:CreateFontString(nil, "OVERLAY")
	end
	self.PriorityFrame.topText:SetTextColor(1,1,1,1)
	self.PriorityFrame.topText:SetFont(media:Fetch("font", self.db.char.barfont), self.db.char.barfontsize)
	self.PriorityFrame.topText:SetPoint("TOP", self.PriorityFrame, "TOP", 0, self.db.char.barfontsize + 2)
	self.PriorityFrame.topText:SetText(L["Next Priority"])
	self:CreateComboPointFrames()
	
	if self.db.char.priority.show and InCombatLockdown() then
		self.PriorityFrame:Show()
	else
		self.PriorityFrame:Hide()
	end
	if self.db.char.priority.titleshow then
		self.PriorityFrame.topText:Show()
	else
		self.PriorityFrame.topText:Hide()
	end
end

function PriorityQueue:ResetPriority()
	self.PriorityFrame:ClearAllPoints()
	self.db.char.priority.point = self.defaults.char.priority.point
	self.db.char.priority.relativeTo = self.defaults.char.priority.relativeTo 
	self.db.char.priority.relativePoint = self.defaults.char.priority.relativePoint
	self.db.char.priority.xOffset = self.defaults.char.priority.xOffset
	self.db.char.priority.yOffset = self.defaults.char.priority.yOffset
	self.db.char.priority.fWidth = self.defaults.char.priority.fWidth
	self.db.char.priority.fHeight = self.defaults.char.priority.fHeight
	self.db.char.priority.scale = self.defaults.char.priority.scale
	self.db.char.priority.option = self.defaults.char.priority.option
	self.PriorityFrame:SetPoint(self.db.char.priority.point, self.db.char.priority.relativeTo, self.db.char.priority.relativePoint, self.db.char.priority.xOffset, self.db.char.priority.yOffset)
	self:CreatePriorityFrame()
	self:Print(L["priority_reset"])
end

function PriorityQueue:SetPriorityIcon(priority)
	self:DebugPrint("setting icon to "..priority)
	if InCombatLockdown() or priority == "none" then
		local icon = self.priorityTable.icon[priority]
		if self:CheckWindShock() then
			icon = C["Wind Shock Icon"]
			self.db.char.priority.next = "wind"
		end
		if self:CheckShamanisticRage() then
			icon = C["Shamanistic Rage Icon"]
			self.db.char.priority.next = "mana"
		end
		-- check if we should show shamanistic rage icon
		self:SetPriorityBackdrop(icon)
	end
end


function PriorityQueue:SetNextPriority()
	if self.db.char.priority.combopoints then
		self:ShowComboPoints()
	end
	if self.priorityTable.test[self.db.char.priority.option[1]]() then
		self.db.char.priority.next = self.db.char.priority.option[1]
	elseif self.priorityTable.test[self.db.char.priority.option[2]]() then
		self.db.char.priority.next = self.db.char.priority.option[2]
	elseif self.priorityTable.test[self.db.char.priority.option[3]]() then
		self.db.char.priority.next = self.db.char.priority.option[3]
	elseif self.priorityTable.test[self.db.char.priority.option[4]]() then
		self.db.char.priority.next = self.db.char.priority.option[4]
	elseif self.priorityTable.test[self.db.char.priority.option[5]]() then
		self.db.char.priority.next = self.db.char.priority.option[5]
	elseif self.priorityTable.test[self.db.char.priority.option[6]]() then
		self.db.char.priority.next = self.db.char.priority.option[6]
	elseif self.priorityTable.test[self.db.char.priority.option[7]]() then
		self.db.char.priority.next = self.db.char.priority.option[7]
	elseif self.priorityTable.test[self.db.char.priority.option[8]]() then
		self.db.char.priority.next = self.db.char.priority.option[8]
	else
		self.db.char.priority.next = "none"
	end	
end

--------------------------
-- Combo Points Frame
--------------------------

function PriorityQueue:ShowComboPoints()
	local col = self.db.char.colours.maelstrom
	for index = 1, 5 do
		if self.PriorityFrame.combo[index].frame then
			if self.db.char.msstacks >= index then
				self.PriorityFrame.combo[index].frame:SetBackdropColor(1, 0, 0, 1)
			else
				self.PriorityFrame.combo[index].frame:SetBackdropColor(col.r, col.g, col.b, 0)
			end
		else
			self:Print("Error could not find combo frame "..index)
		end
	end
end

function PriorityQueue:CreateComboPointFrames()
	self.PriorityFrame.combo = {}
	for index = 1, 5 do 
		self:CreateComboPoint(index)
	end
end
	
function PriorityQueue:CreateComboPoint(index)
	if not self.PriorityFrame.combo[index] or not self.PriorityFrame.combo[index].frame then
		self.PriorityFrame.combo[index] = {}
		self.PriorityFrame.combo[index].frame = CreateFrame("Frame", "SAA_PriorityComboFrame"..index, self.PriorityFrame)
	end
	local comboFrame = self.PriorityFrame.combo[index].frame
	local width = self.db.char.priority.fWidth / 5
	local height = self.db.char.priority.fHeight / 5
	comboFrame:SetScale(self.db.char.priority.scale)
	comboFrame:SetFrameStrata("BACKGROUND")
	comboFrame:SetWidth(width)
	comboFrame:SetHeight(height)
	comboFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 4,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})	
	local col = self.db.char.colours.maelstrom
	comboFrame:SetBackdropColor(0, 0, 0, 0);
	comboFrame:SetPoint("BOTTOMLEFT",  self.PriorityFrame, "BOTTOMLEFT", (index - 1) * width , - height)

	if self.db.char.priority.combopoints then
		comboFrame:Show()
	else
		comboFrame:Hide()
	end
end

---------------------------
-- functions
---------------------------

function PriorityQueue:DisplayVersion()
	self:Print(self.version)
end

function PriorityQueue:GetGCD(spell)
	local startTime, duration, enabled = GetSpellCooldown(spell) 
	duration = duration or 0 -- force nil to 0 if spell isnt available.
	if duration > 0 then 
		return startTime + duration - GetTime()
	end
	return 0
end

function PriorityQueue:CreateMsgFrame()
	self.msgFrame:ClearAllPoints()
	self.msgFrame:SetWidth(400)
	self.msgFrame:SetHeight(75)
	self.msgFrame:SetFrameStrata("BACKGROUND")
	self.msgFrame:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		--edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 12,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	self.msgFrame:SetBackdropColor(1, 1, 1, 0)
	self.msgFrame:SetMovable(true)
	self.msgFrame:RegisterForDrag("LeftButton")
	self.msgFrame:SetScript("OnDragStart", 
		function()
			self.msgFrame:StartMoving();
		end );
	self.msgFrame:SetScript("OnDragStop",
		function()
			self.msgFrame:StopMovingOrSizing();
			self.msgFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char.warning, self.msgFrame);
		end );
	self.msgFrame:SetPoint(self.db.char.warning.point, self.db.char.warning.relativeTo, self.db.char.warning.relativePoint, self.db.char.warning.xOffset, self.db.char.warning.yOffset)
	self.msgFrame:SetInsertMode("TOP")
	self.msgFrame:SetFrameStrata("HIGH")
	self.msgFrame:SetToplevel(true)
	local font = media:Fetch("font", self.db.char.msgfont)
	self.msgFrame:SetFont(font, self.db.char.msgfontsize, self.db.char.msgfonteffect)
		
	self.msgFrame:Show()
end

function PriorityQueue:PrintMsg(msg, col, time)
	if self.db.char.warning.show then
		if col == nil then
			col = { r=1, b=1, g=1, a=1 }
		end
		if time == nil then 
			time = 3
		end
		if time ~= 5 then
			self.msgFrame:SetTimeVisible(time)
		end
		self.msgFrame:AddMessage(msg, col.r, col.g, col.b, 1, col.time)
	end
end

function PriorityQueue:DebugPrint(msg)
	if self.db.char.debug then
		self:Print(msg)
	end
end